@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -41,10 +40,16 @@ typedef struct _Component Component;
 struct _Component {
   Element element;
 
-  ConnectionPoint connections[8];
+  ConnectionPoint connections[10];
 
   char *stereotype;
   Text *text;
+
+  char *st_stereotype;
+  TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 #define COMPONENT_BORDERWIDTH 0.1
@@ -55,37 +60,36 @@ struct _Component {
 
 static real component_distance_from(Component *cmp, Point *point);
 static void component_select(Component *cmp, Point *clicked_point,
-				Renderer *interactive_renderer);
-static void component_move_handle(Component *cmp, Handle *handle,
-				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void component_move(Component *cmp, Point *to);
-static void component_draw(Component *cmp, Renderer *renderer);
-static Object *component_create(Point *startpoint,
+				DiaRenderer *interactive_renderer);
+static ObjectChange* component_move_handle(Component *cmp, Handle *handle,
+					   Point *to, ConnectionPoint *cp,
+					   HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* component_move(Component *cmp, Point *to);
+static void component_draw(Component *cmp, DiaRenderer *renderer);
+static DiaObject *component_create(Point *startpoint,
 				   void *user_data,
 				   Handle **handle1,
 				   Handle **handle2);
 static void component_destroy(Component *cmp);
-static Object *component_copy(Component *cmp);
-
-static void component_save(Component *cmp, ObjectNode obj_node,
-			      const char *filename);
-static Object *component_load(ObjectNode obj_node, int version,
+static DiaObject *component_load(ObjectNode obj_node, int version,
 				 const char *filename);
 
 static PropDescription *component_describe_props(Component *component);
-static void component_get_props(Component *component, Property *props, guint nprops);
-static void component_set_props(Component *component, Property *props, guint nprops);
+static void component_get_props(Component *component, GPtrArray *props);
+static void component_set_props(Component *component, GPtrArray *props);
 
 static void component_update_data(Component *cmp);
 
 static ObjectTypeOps component_type_ops =
 {
   (CreateFunc) component_create,
-  (LoadFunc)   component_load,
-  (SaveFunc)   component_save
+  (LoadFunc)   component_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType component_type =
+DiaObjectType component_type =
 {
   "UML - Component",   /* name */
   0,                      /* version */
@@ -99,7 +103,7 @@ static ObjectOps component_ops = {
   (DrawFunc)            component_draw,
   (DistanceFunc)        component_distance_from,
   (SelectFunc)          component_select,
-  (CopyFunc)            component_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            component_move,
   (MoveHandleFunc)      component_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -112,153 +116,102 @@ static ObjectOps component_ops = {
 
 static PropDescription component_props[] = {
   ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
   N_("Stereotype"), NULL, NULL },
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
   PROP_STD_TEXT_COLOUR,
-  PROP_STD_TEXT,
-  
+  { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL },   
   PROP_DESC_END
 };
 
 static PropDescription *
 component_describe_props(Component *component)
 {
-  if (component_props[0].quark == 0)
+  if (component_props[0].quark == 0) {
     prop_desc_list_calculate_quarks(component_props);
+  }
   return component_props;
 }
 
 static PropOffset component_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
-  /*{ "stereotype", PROP_TYPE_STRING, offsetof(Component , stereotype) },*/
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Component,line_color)},
+  {"fill_colour",PROP_TYPE_COLOUR,offsetof(Component,fill_color)},
+  {"stereotype", PROP_TYPE_STRING, offsetof(Component , stereotype) },
+  {"text",PROP_TYPE_TEXT,offsetof(Component,text)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Component,attrs.font)},
+  {"text_height",PROP_TYPE_REAL,offsetof(Component,attrs.height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Component,attrs.color)},  
   { NULL, 0, 0 },
 };
 
-static struct { const gchar *name; GQuark q; } quarks[] = {
-  { "text_font" },
-  { "text_height" },
-  { "text_colour" },
-  { "text" },
-  { "stereotype" }
-};
 
 static void
-component_get_props(Component * component, Property *props, guint nprops)
+component_get_props(Component * component, GPtrArray *props)
 {
-  guint i;
-
-  if (object_get_props_from_offsets(&component->element.object, 
-                                    component_offsets, props, nprops))
-    return;
-  /* these props can't be handled as easily */
-  if (quarks[0].q == 0)
-    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-      quarks[i].q = g_quark_from_static_string(quarks[i].name);
-  for (i = 0; i < nprops; i++) {
-    GQuark pquark = g_quark_from_string(props[i].name);
-
-    if (pquark == quarks[0].q) {
-      props[i].type = PROP_TYPE_FONT;
-      PROP_VALUE_FONT(props[i]) = component->text->font;
-    } else if (pquark == quarks[1].q) {
-      props[i].type = PROP_TYPE_REAL;
-      PROP_VALUE_REAL(props[i]) = component->text->height;
-    } else if (pquark == quarks[2].q) {
-      props[i].type = PROP_TYPE_COLOUR;
-      PROP_VALUE_COLOUR(props[i]) = component->text->color;
-    } else if (pquark == quarks[3].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      PROP_VALUE_STRING(props[i]) = text_get_string_copy(component->text);
-    } else if (pquark == quarks[4].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      if (component->stereotype &&
-	  component->stereotype[0] != '\0')
-	PROP_VALUE_STRING(props[i]) =
-	  stereotype_to_string(component->stereotype);
-      else
-	PROP_VALUE_STRING(props[i]) = NULL;
-    }
-  }
+  text_get_attributes(component->text,&component->attrs);
+  object_get_props_from_offsets(&component->element.object,
+                                component_offsets,props);
 }
 
 static void
-component_set_props(Component *component, Property *props, guint nprops)
+component_set_props(Component *component, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&component->element.object, 
-                                     component_offsets, props, nprops)) {
-    guint i;
-
-    if (quarks[0].q == 0)
-      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-	quarks[i].q = g_quark_from_static_string(quarks[i].name);
-
-    for (i = 0; i < nprops; i++) {
-      GQuark pquark = g_quark_from_string(props[i].name);
-
-      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
-	text_set_font(component->text, PROP_VALUE_FONT(props[i]));
-      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
-	text_set_height(component->text, PROP_VALUE_REAL(props[i]));
-      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
-	text_set_color(component->text, &PROP_VALUE_COLOUR(props[i]));
-      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
-	text_set_string(component->text, PROP_VALUE_STRING(props[i]));
-      } else if (pquark == quarks[4].q && props[i].type == PROP_TYPE_STRING) {
-	g_free(component->stereotype);
-	if (PROP_VALUE_STRING(props[i]) &&
-	    PROP_VALUE_STRING(props[i])[0] != '\0')
-	  component->stereotype =
-	    string_to_stereotype(PROP_VALUE_STRING(props[i]));
-	else 
-	  component->stereotype = NULL;
-      }
-    }
-  }
+  object_set_props_from_offsets(&component->element.object, 
+                                component_offsets, props);
+  apply_textattr_properties(props,component->text,"text",&component->attrs);
+  g_free(component->st_stereotype);
+  component->st_stereotype = NULL;
   component_update_data(component);
 }
 
 static real
 component_distance_from(Component *cmp, Point *point)
 {
-  Object *obj = &cmp->element.object;
+  DiaObject *obj = &cmp->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
 static void
 component_select(Component *cmp, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   text_set_cursor(cmp->text, clicked_point, interactive_renderer);
   text_grab_focus(cmp->text, &cmp->element.object);
   element_update_handles(&cmp->element);
 }
 
-static void
+static ObjectChange*
 component_move_handle(Component *cmp, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		      Point *to, ConnectionPoint *cp,
+		      HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(cmp!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 component_move(Component *cmp, Point *to)
 {
   cmp->element.corner = *to;
 
   component_update_data(cmp);
+
+  return NULL;
 }
 
 static void
-component_draw(Component *cmp, Renderer *renderer)
+component_draw(Component *cmp, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point p1, p2;
@@ -273,48 +226,49 @@ component_draw(Component *cmp, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, COMPONENT_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, COMPONENT_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   p1.x = x + COMPONENT_CWIDTH/2; p1.y = y;
   p2.x = x+w; p2.y = y+h;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
-  renderer->ops->draw_rect(renderer, 
+			   &cmp->fill_color);
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &cmp->line_color);
 
   p1.x= x; p1.y = y +(h - 3*COMPONENT_CHEIGHT)/2.0;
   p2.x = x+COMPONENT_CWIDTH; p2.y = p1.y + COMPONENT_CHEIGHT;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &cmp->fill_color);
   
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &cmp->line_color);
   
   p1.y = p2.y + COMPONENT_CHEIGHT;
   p2.y = p1.y + COMPONENT_CHEIGHT;
 
-  renderer->ops->fill_rect(renderer, 
+  renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &cmp->fill_color);
   
-  renderer->ops->draw_rect(renderer, 
+  renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &cmp->line_color);
 
-  if (cmp->stereotype != NULL &&
-      cmp->stereotype[0] != '\0') {
+  if (cmp->st_stereotype != NULL &&
+      cmp->st_stereotype[0] != '\0') {
     p1 = cmp->text->position;
     p1.y -= cmp->text->height;
-    renderer->ops->draw_string(renderer, cmp->stereotype, &p1, 
-			       ALIGN_LEFT, &color_black);
+    renderer_ops->set_font(renderer, cmp->text->font, cmp->text->height);
+    renderer_ops->draw_string(renderer, cmp->st_stereotype, &p1, 
+			       ALIGN_LEFT, &cmp->attrs.color);
   }
 
   text_draw(cmp->text, renderer);
@@ -324,9 +278,16 @@ static void
 component_update_data(Component *cmp)
 {
   Element *elem = &cmp->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   Point p;
+  real cw2, ch;
 
+  cmp->stereotype = remove_stereotype_from_string(cmp->stereotype);
+  if (!cmp->st_stereotype) {
+    cmp->st_stereotype =  string_to_stereotype(cmp->stereotype);
+  }
+
+  text_calc_boundingbox(cmp->text, NULL);
   elem->width = cmp->text->max_width + 2*COMPONENT_MARGIN_X + COMPONENT_CWIDTH;
   elem->width = MAX(elem->width, 2*COMPONENT_CWIDTH);
   elem->height =  cmp->text->height*cmp->text->numlines +
@@ -338,37 +299,65 @@ component_update_data(Component *cmp)
   p.y += COMPONENT_CHEIGHT;
   p.y += cmp->text->ascent;
   if (cmp->stereotype &&
-      cmp->stereotype[0] != '\0')
+      cmp->stereotype[0] != '\0') {
     p.y += cmp->text->height;
+  }
   text_set_position(cmp->text, &p);
 
-  if (cmp->stereotype &&
-      cmp->stereotype[0] != '\0') {
-    Font *font;
+  if (cmp->st_stereotype &&
+      cmp->st_stereotype[0] != '\0') {
+    DiaFont *font;
     font = cmp->text->font;
     elem->height += cmp->text->height;
-    elem->width = MAX(elem->width, font_string_width(cmp->stereotype,
+    elem->width = MAX(elem->width, dia_font_string_width(cmp->st_stereotype,
 						     font, cmp->text->height) +
 		      2*COMPONENT_MARGIN_X + COMPONENT_CWIDTH);
   }
 
+  cw2 = COMPONENT_CWIDTH/2;
+  ch = COMPONENT_CHEIGHT;
   /* Update connections: */
-  cmp->connections[0].pos = elem->corner;
-  cmp->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
-  cmp->connections[1].pos.y = elem->corner.y;
-  cmp->connections[2].pos.x = elem->corner.x + elem->width;
-  cmp->connections[2].pos.y = elem->corner.y;
-  cmp->connections[3].pos.x = elem->corner.x;
-  cmp->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
-  cmp->connections[4].pos.x = elem->corner.x + elem->width;
-  cmp->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
-  cmp->connections[5].pos.x = elem->corner.x;
-  cmp->connections[5].pos.y = elem->corner.y + elem->height;
-  cmp->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
-  cmp->connections[6].pos.y = elem->corner.y + elem->height;
-  cmp->connections[7].pos.x = elem->corner.x + elem->width;
-  cmp->connections[7].pos.y = elem->corner.y + elem->height;
-  
+  connpoint_update(&cmp->connections[0],
+		   elem->corner.x + cw2,
+		   elem->corner.y,
+		   DIR_NORTH|DIR_WEST);
+  connpoint_update(&cmp->connections[1],
+		   elem->corner.x + cw2 + (elem->width - cw2) / 2,
+		   elem->corner.y,
+		   DIR_NORTH);
+  connpoint_update(&cmp->connections[2],
+		   elem->corner.x + elem->width,
+		   elem->corner.y,
+		   DIR_NORTH|DIR_EAST);
+  connpoint_update(&cmp->connections[3],
+		   elem->corner.x + cw2,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_WEST);
+  connpoint_update(&cmp->connections[4],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height / 2.0,
+		   DIR_EAST);
+  connpoint_update(&cmp->connections[5],
+		   elem->corner.x + cw2,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH|DIR_WEST);
+  connpoint_update(&cmp->connections[6],
+		   elem->corner.x + cw2 + (elem->width - cw2)/2,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH);
+  connpoint_update(&cmp->connections[7],
+		   elem->corner.x + elem->width,
+		   elem->corner.y + elem->height,
+		   DIR_SOUTH|DIR_EAST);
+  connpoint_update(&cmp->connections[8],
+		   elem->corner.x,
+		   elem->corner.y + elem->height / 2.0 - ch,
+		   DIR_WEST);
+  connpoint_update(&cmp->connections[9],
+		   elem->corner.x,
+		   elem->corner.y + elem->height / 2.0 + ch,
+		   DIR_WEST);
+
   element_update_boundingbox(elem);
 
   obj->position = elem->corner;
@@ -376,7 +365,7 @@ component_update_data(Component *cmp)
   element_update_handles(elem);
 }
 
-static Object *
+static DiaObject *
 component_create(Point *startpoint,
 		    void *user_data,
 		    Handle **handle1,
@@ -384,9 +373,9 @@ component_create(Point *startpoint,
 {
   Component *cmp;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
-  Font *font;
+  DiaFont *font;
   int i;
   
   cmp = g_malloc0(sizeof(Component));
@@ -397,25 +386,32 @@ component_create(Point *startpoint,
 
   obj->ops = &component_ops;
 
-  elem->corner = *startpoint;
+  obj->can_parent = TRUE;
 
-  font = font_getfont("Helvetica");
+  elem->corner = *startpoint;
+  cmp->line_color = attributes_get_foreground();
+  cmp->fill_color = attributes_get_background();
+
+  font = dia_font_new_from_style (DIA_FONT_SANS, 0.8);
   p = *startpoint;
   p.x += COMPONENT_CWIDTH + COMPONENT_MARGIN_X;
   p.y += 2*COMPONENT_CHEIGHT;
   
   cmp->text = new_text("", font, 0.8, &p, &color_black, ALIGN_LEFT);
+  text_get_attributes(cmp->text,&cmp->attrs);
+
+  dia_font_unref(font);
   
-  element_init(elem, 8, 8);
+  element_init(elem, 8, 10);
   
-  for (i=0;i<8;i++) {
+  for (i=0;i<10;i++) {
     obj->connections[i] = &cmp->connections[i];
     cmp->connections[i].object = obj;
     cmp->connections[i].connected = NULL;
   }
   elem->extra_spacing.border_trans = COMPONENT_BORDERWIDTH/2.0;
   cmp->stereotype = NULL;
-
+  cmp->st_stereotype = NULL;
   component_update_data(cmp);
 
   for (i=0;i<8;i++) {
@@ -431,101 +427,16 @@ static void
 component_destroy(Component *cmp)
 {
   text_destroy(cmp->text);
-
-  element_destroy(&cmp->element);
   g_free(cmp->stereotype);
+  g_free(cmp->st_stereotype);
+  element_destroy(&cmp->element);
 }
 
-static Object *
-component_copy(Component *cmp)
-{
-  int i;
-  Component *newcmp;
-  Element *elem, *newelem;
-  Object *newobj;
-  
-  elem = &cmp->element;
-  
-  newcmp = g_malloc0(sizeof(Component));
-  newelem = &newcmp->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-
-  newcmp->text = text_copy(cmp->text);
-  
-  for (i=0;i<8;i++) {
-    newobj->connections[i] = &newcmp->connections[i];
-    newcmp->connections[i].object = newobj;
-    newcmp->connections[i].connected = NULL;
-    newcmp->connections[i].pos = cmp->connections[i].pos;
-    newcmp->connections[i].last_pos = cmp->connections[i].last_pos;
-  }
-  component_update_data(newcmp);
-  if (cmp->stereotype != NULL)
-    newcmp->stereotype = g_strdup(cmp->stereotype);
-  else
-    newcmp->stereotype = NULL;
-  
-  return &newcmp->element.object;
-}
-
-
-static void
-component_save(Component *cmp, ObjectNode obj_node,
-		  const char *filename)
-{
-  element_save(&cmp->element, obj_node);
-
-  data_add_text(new_attribute(obj_node, "text"),
-		cmp->text);
-  data_add_string(new_attribute(obj_node, "stereotype"),
-		  cmp->stereotype);
-}
-
-static Object *
+static DiaObject *
 component_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Component *cmp;
-  AttributeNode attr;
-  Element *elem;
-  Object *obj;
-  int i;
-  
-  cmp = g_malloc0(sizeof(Component));
-  elem = &cmp->element;
-  obj = &elem->object;
-  
-  obj->type = &component_type;
-  obj->ops = &component_ops;
-
-  element_load(elem, obj_node);
-  
-  cmp->text = NULL;
-  attr = object_find_attribute(obj_node, "text");
-  if (attr != NULL)
-    cmp->text = data_text(attribute_first_data(attr));
-  
-  cmp->stereotype = NULL;
-  attr = object_find_attribute(obj_node, "stereotype");
-  if (attr != NULL)
-    cmp->stereotype = data_string(attribute_first_data(attr));
-
-  element_init(elem, 8, 8);
-
-  for (i=0;i<8;i++) {
-    obj->connections[i] = &cmp->connections[i];
-    cmp->connections[i].object = obj;
-    cmp->connections[i].connected = NULL;
-  }
-  elem->extra_spacing.border_trans = COMPONENT_BORDERWIDTH/2.0;
-  component_update_data(cmp);
-
-  for (i=0;i<8;i++) {
-    obj->handles[i]->type = HANDLE_NON_MOVABLE;
-  }
-
-  return &cmp->element.object;
+  return object_load_using_properties(&component_type,
+                                      obj_node,version,filename);
 }
 
 
