@@ -50,6 +50,10 @@ struct _Objet {
   char *exstate;  /* used for explicit state */
   Text *attributes;
 
+  Color text_color;
+  Color line_color;
+  Color fill_color;
+
   Point ex_pos, st_pos;
   int is_active;
   int show_attributes;
@@ -76,12 +80,12 @@ static ObjectChange* objet_move_handle(Objet *ob, Handle *handle,
 				       HandleMoveReason reason, ModifierKeys modifiers);
 static ObjectChange* objet_move(Objet *ob, Point *to);
 static void objet_draw(Objet *ob, DiaRenderer *renderer);
-static Object *objet_create(Point *startpoint,
+static DiaObject *objet_create(Point *startpoint,
 			    void *user_data,
 			    Handle **handle1,
 			    Handle **handle2);
 static void objet_destroy(Objet *ob);
-static Object *objet_load(ObjectNode obj_node, int version,
+static DiaObject *objet_load(ObjectNode obj_node, int version,
 			  const char *filename);
 static PropDescription *objet_describe_props(Objet *objet);
 static void objet_get_props(Objet *objet, GPtrArray *props);
@@ -98,7 +102,7 @@ static ObjectTypeOps objet_type_ops =
 };
 
 /* Non-nice typo, needed for backwards compatibility. */
-ObjectType objet_type =
+DiaObjectType objet_type =
 {
   "UML - Objet",   /* name */
   0,                      /* version */
@@ -107,7 +111,7 @@ ObjectType objet_type =
   &objet_type_ops       /* ops */
 };
 
-ObjectType umlobject_type =
+DiaObjectType umlobject_type =
 {
   "UML - Object",   /* name */
   0,                      /* version */
@@ -135,10 +139,9 @@ static ObjectOps objet_ops = {
 
 static PropDescription objet_props[] = {
   ELEMENT_COMMON_PROPERTIES,
-  /*PROP_STD_TEXT_ALIGNMENT,
-  PROP_STD_TEXT_FONT,
-  PROP_STD_TEXT_HEIGHT,
-  PROP_STD_TEXT_COLOUR,*/
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL },
   { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
     N_("Stereotype"), NULL, NULL },
@@ -153,7 +156,7 @@ static PropDescription objet_props[] = {
     N_("Show attributes"),NULL,NULL}, 
   { "multiple", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
     N_("Multiple instance"),NULL,NULL}, 
-  { NULL, 0, 0, NULL, NULL, NULL, 0}
+  PROP_DESC_END
 };
 
 static PropDescription *
@@ -167,6 +170,9 @@ objet_describe_props(Objet *ob)
 
 static PropOffset objet_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "text_colour",PROP_TYPE_COLOUR,offsetof(Objet, text_color) },
+  { "line_colour",PROP_TYPE_COLOUR,offsetof(Objet, line_color) },
+  { "fill_colour",PROP_TYPE_COLOUR,offsetof(Objet, fill_color) },
   { "name", PROP_TYPE_STRING, offsetof(Objet, exstate) },
   { "stereotype", PROP_TYPE_STRING, offsetof(Objet, stereotype) },
   { "text", PROP_TYPE_TEXT, offsetof(Objet, text) },
@@ -203,7 +209,7 @@ objet_set_props(Objet *objet, GPtrArray *props)
 static real
 objet_distance_from(Objet *ob, Point *point)
 {
-  Object *obj = &ob->element.object;
+  DiaObject *obj = &ob->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
@@ -273,10 +279,10 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
     p2.y -= OBJET_MARGIN_M;
     renderer_ops->fill_rect(renderer, 
 			     &p1, &p2,
-			     &color_white); 
+			     &ob->fill_color);
     renderer_ops->draw_rect(renderer, 
 			     &p1, &p2,
-			     &color_black);
+			     &ob->line_color);
     p1.x -= OBJET_MARGIN_M;
     p1.y += OBJET_MARGIN_M;
     p2.x -= OBJET_MARGIN_M;
@@ -286,10 +292,10 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
     
   renderer_ops->fill_rect(renderer, 
 			   &p1, &p2,
-			   &color_white);
+			   &ob->fill_color);
   renderer_ops->draw_rect(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &ob->line_color);
 
   
   text_draw(ob->text, renderer);
@@ -298,14 +304,14 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
       renderer_ops->draw_string(renderer,
 				 ob->st_stereotype,
 				 &ob->st_pos, ALIGN_CENTER,
-				 &color_black);
+				 &ob->text_color);
   }
 
   if ((ob->exstate != NULL) && (ob->exstate[0] != '\0')) {
       renderer_ops->draw_string(renderer,
 				 ob->exstate,
 				 &ob->ex_pos, ALIGN_CENTER,
-				 &color_black);
+				 &ob->text_color);
   }
 
   /* Is there a better way to underline? */
@@ -321,7 +327,7 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
     p2.x = p1.x + ob->text->row_width[i];
     renderer_ops->draw_line(renderer,
 			     &p1, &p2,
-			     &color_black);
+			     &ob->text_color);
     p1.y = p2.y += ob->text->height;
   }
 
@@ -332,7 +338,7 @@ objet_draw(Objet *ob, DiaRenderer *renderer)
       renderer_ops->set_linewidth(renderer, bw);
       renderer_ops->draw_line(renderer,
 			       &p1, &p2,
-			       &color_black);
+			       &ob->line_color);
 
       text_draw(ob->attributes, renderer);
   }
@@ -342,7 +348,7 @@ static void
 objet_update_data(Objet *ob)
 {
   Element *elem = &ob->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   DiaFont *font;
   Point p1, p2;
   real h, w = 0;
@@ -437,7 +443,7 @@ objet_update_data(Objet *ob)
   element_update_handles(elem);
 }
 
-static Object *
+static DiaObject *
 objet_create(Point *startpoint,
 		    void *user_data,
 		    Handle **handle1,
@@ -445,7 +451,7 @@ objet_create(Point *startpoint,
 {
   Objet *ob;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
   DiaFont *font;
   int i;
@@ -460,6 +466,10 @@ objet_create(Point *startpoint,
 
   elem->corner = *startpoint;
 
+  ob->text_color = color_black;
+  ob->line_color = attributes_get_foreground();
+  ob->fill_color = attributes_get_background();
+
   font = dia_font_new_from_style(DIA_FONT_SANS, OBJET_FONTHEIGHT);
   
   ob->show_attributes = FALSE;
@@ -513,7 +523,7 @@ objet_destroy(Objet *ob)
   element_destroy(&ob->element);
 }
 
-static Object *
+static DiaObject *
 objet_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&objet_type,
