@@ -20,7 +20,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 #include "intl.h"
 #include "filter.h"
 #include "plug-ins.h"
